---
title: TP Physique, Système Solaire
author:
  - Boris STEFANOVIC
  - Michael CERQUEIRA DA COSTA
date: 2021-12-21
...


# Théorie et Calculs

## Forces
Chaque corps subit les forces d'attraction des autres corps présents.
On a la formule pour calculer la force entre deux masses:
$$F = \frac{G \times M \times m}{r^2}$$
La force résultante sur la masse $m$ est donc:
$$F_{res \rightarrow m} = \sum_{i \in autres corps} \frac{G \times M_i \times m}{r^2}$$
On applique la deuxième loi de Newton:
$$F = m \times a$$
Toujours avec $m$, la masse du corps qui subit la force, on pose, par
combinaison des deux formules précédentes:
$$m \times a = \frac{G \times M \times m}{r^2}$$
On simplifie par la masse du corps étudié, comme suit:
$$a = \frac{G \times M}{r^2}$$
Ainsi, on peut directement calculer le vecteur accélération "résultant":
$$a_{res \rightarrow m} = \sum_{i \in autres corps} \frac{G \times M_i}{r^2}$$

## Calcul de l'Accélération
C'est en utilisant cette la formule précédente que le vecteur accélération (au temps $t$)
a été calculé, de façon itérative, par une suite d'additions de vecteurs.
Ainsi, notre programme ne comporte aucune représentation de vecteur force.

## Calcul de la Vitesse
Pour calculer la position suivante de la planète lors de chaque itération,
nous avons la formule suivante, qui exprime la position en fonction du temps:
$$
\overrightarrow{x}(t + \Delta t) =
\overrightarrow{x_{prec}} +
\Delta t \overrightarrow{v}(t) +
\frac{(\Delta t)^2}{2} \overrightarrow{a}(t)
$$



# Analyse des Résultats

## Simulation
De par l'utilisation d'un intervalle de temps, l'évolution du système n'est pas continue.
Cela diffère avec ce qu'on peut observer dans la réalité. On perd donc en précision à ce niveau-là.

## Observations et Vérifications
Nous avons pu constater que les planètes interagissent
entre elles en plus de subir l'attraction de l'étoile au centre du système.
Ceci a pu être observé lorsqu'une planète s'est vue accélérer **à l'aphélie**,
lors du passage d'une planète fictive relativement massive à proximité.

## Référentiel
Si on ajoutait des corps suffisamment massifs dans nos tableaux de planètes,
la réalité voudrait que l'étoile centrale voie son vecteur vitesse changer
en fonction des autres corps présents.
Pourtant, dans le cadre de ce travail, nous centrons nôtre référentiel sur
cette étoile. Nous ne calculons donc jamais les modifications de son vecteur vitesse.
Les planètes gravitant autour de cette dernière, aussi massives soient elles
(test effectué avec une masse proche d'un dixième de celle du soleil),
apparaîtront comme tournant autour d'un point fixe.

![le système en cours de simulation](system.png)


# L'Aspect Informatique

## Décomposition du Problème
Nous avons adopté une approche de bas en haut ("bottom-to-top").
Nous avons décomposé le problème aussi précisément que possible
en nous inspirant des prototypes proposés avant d'écrire la moindre ligne de code.

## Adaptation du code Existant
Au fichiers de `vec2`, nous avons ajouté la méthode
`vec2_to_coordinates_centered(vec2 v, uint32_t width, uint32_t height)`
qui agit de façon similaire à
`vec2_to_coordinates(vec2 v, uint32_t width, uint32_t height)`
mais fait correspondre au vecteur (0,0) le centre de l'écran.
Ainsi, en plus d'accepter des vecteurs de norme comprise dans \[0;1\],
elle accepte des arguments de norme dans \[-1;+1\], ces dernières étant facteurs
de demi-dimensions de l'écran en partant du centre de l'écran (l'origine).
En plus de cela, une fonction
`vec2 calc_prev_position(const double star_mass, const vec2 pos, const double ecc, const double axm, const double delta_t)`
a été implémentée pour séparer le calcul de la vitesse initiale des planètes du reste du code.

## Recueil des valeurs physiques relatives aux planètes
Nous avons créé le fichier **constants.h** pour contenir les valeurs des
**masses, périhélies, excentricités et demi-grands-axes**
de chaque planète étudiée.
Ces valeurs ont été recueillies sur internet et dans les tables CRM.

![recueil des constantes réelles](constants.png){ width=40% margin=auto }

## Ajout de planètes fictives
Une fois le système suffisamment robuste, nous avons ajouté quatre planètes
à notre système. Ces dernières interagissent avec tous les autres objets de
la simulation, comme il se doit, mais sont séparées au niveau informatique.
Ainsi, il serait facile de revenir à un système plus proche de la réalité en
ne modifiant que peu de lignes de code.
Toutes les planètes fictives sont affichées en vert, pour les dissocier des
planètes réelles.

## Déroulement du Programme
Une fois l'initialisation terminée, la simulation est lancée.
A chaque itération, qui représente le passage du temps $\Delta t$,
la position de chaque planète est mise à jour.
Après avoir executé tous ces calculs, le corps de la boucle principale se termine par
l'affichage du système, de façon simplifiée
(les rayons des planètes et de l'étoile ne sont pas à l'échelle par souci de
visibilité et facilité de lecture par l'utilisateur).

## Tests
Nous avons obtenu un système fonctionnel. Ce dernier a été testé pendant plus d'une heure
(avec `DELTA_T = 8000.0`) sans produire de résultat qui semblait physiquement incohérent.


# Réflexions

## Passage du continu au discret
Si les grandeurs physiques avec lesquelles on travaille sont continues,
leur représentation en mémoire d'un ordinateur est limitée en précision.
Même si dans le cadre de ce travail, la précision fournie par les `double`
est suffisante pour ne pas avoir à s'en inquiéter, nous avons essayé de
minimiser le nombre d'opérations effectuées pour le calcul de chaque valeur.
De plus, l'ajout de quelques préventions de divisions par zéro s'est aussi
avéré nécessaire pour la bonne execution du programme.

## Erreurs de signe
Lors de la première compilation et simulation, nous avons noté que les planètes
s'éloignaient du soleil, à l'infini. Cela n'étant pas représentatif de la réalité,
la cause de cette perturbation n'était autre qu'une inversion malheureuse
du signe du vecteur accélération. Une fois le bug identifié, la simulation
a été corrigée dans la minute.

## Améliorations Logicielles Possibles
En ce qui concerne les structures de données utilisées, peut être que l'utilisation
de listes chaînées au lieu de tableaux pourrait faciliter la modification des
paramètres du système, sans augmenter le temps d'execution *théorique*
(en pratique, les tableaux restent plus efficaces).
Les données de l'étoile et des planètes, quant à elles devraient être chargées depuis
un fichier de paramètres ou à la rigueur, l'entrée standard (utilisable avec un "pipe"),
pour des questions de design.
